@@ -523,7 +523,9 @@ void ReadSpecialSettings (const char * name)
   settings.hacks = 0;
 
   //detect games which require special hacks
-  if (strstr(name, (const char *)"ZELDA") || strstr(name, (const char *)"MASK"))
+  if (strstr(name, (const char *)"ZELDA"))
+    settings.hacks |= (hack_Zelda | hack_OoT);
+  else if(strstr(name, (const char *)"MASK"))
     settings.hacks |= hack_Zelda;
   else if (strstr(name, (const char *)"ROADSTERS TROPHY"))
     settings.hacks |= hack_Zelda;
@@ -1182,7 +1184,7 @@ int InitGfx ()
     char strSstWinOpenExt[] ="grSstWinOpenExt";
     GRWINOPENEXT grSstWinOpenExt = (GRWINOPENEXT)grGetProcAddress(strSstWinOpenExt);
     if (grSstWinOpenExt)
-      gfx_context = grSstWinOpenExt ((FxU32)NULL,
+      gfx_context = grSstWinOpenExt ((uintptr_t)NULL,
       res_data,
       GR_REFRESH_60Hz,
       GR_COLORFORMAT_RGBA,
@@ -1192,7 +1194,7 @@ int InitGfx ()
       1);   // 1 auxillary buffer
   }
   if (!gfx_context)
-    gfx_context = grSstWinOpen ((FxU32)NULL,
+    gfx_context = grSstWinOpen ((uintptr_t)NULL,
     res_data,
     GR_REFRESH_60Hz,
     GR_COLORFORMAT_RGBA,
@@ -1903,7 +1905,7 @@ EXPORT int CALL RomOpen (void)
 
   /* cxd4 -- Glide64 tries to predict PAL scaling based on the ROM header. */
   region = OS_TV_TYPE_NTSC; /* Invalid region codes are probably NTSC betas. */
-  switch (gfx.HEADER[0x3E ^ 3])
+  switch (gfx.HEADER[BYTEADDR(0x3E)])
   {
      case 'A': /* generic NTSC, not documented, used by 1080 Snowboarding */
         region = OS_TV_TYPE_NTSC; break;
@@ -1952,14 +1954,14 @@ EXPORT int CALL RomOpen (void)
 
   // get the name of the ROM
   for (int i=0; i<20; i++)
-    name[i] = gfx.HEADER[(32+i)^3];
+    name[i] = gfx.HEADER[BYTEADDR(32+i)];
   name[20] = 0;
 
   // remove all trailing spaces
   while (name[strlen(name)-1] == ' ')
     name[strlen(name)-1] = 0;
 
-  strncpy(rdp.RomName, name, sizeof(name));
+  strncpy(rdp.RomName, name, sizeof(rdp.RomName));
   ReadSpecialSettings (name);
   ClearCache ();
 
